"use strict";
/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const util_1 = require("util");
const plugin_discovery_error_1 = require("./plugin_discovery_error");
const fsReadFileAsync = util_1.promisify(fs_1.readFile);
/**
 * Name of the JSON manifest file that should be located in the plugin directory.
 */
const MANIFEST_FILE_NAME = 'kibana.json';
/**
 * The special "kibana" version can be used by the plugins to be always compatible.
 */
const ALWAYS_COMPATIBLE_VERSION = 'kibana';
/**
 * Regular expression used to extract semantic version part from the plugin or
 * kibana version, e.g. `1.2.3` ---> `1.2.3` and `7.0.0-alpha1` ---> `7.0.0`.
 */
const SEM_VER_REGEX = /\d+\.\d+\.\d+/;
/**
 * Tries to load and parse the plugin manifest file located at the provided plugin
 * directory path and produces an error result if it fails to do so or plugin manifest
 * isn't valid.
 * @param pluginPath Path to the plugin directory where manifest should be loaded from.
 * @param packageInfo Kibana package info.
 */
async function parseManifest(pluginPath, packageInfo) {
    const manifestPath = path_1.resolve(pluginPath, MANIFEST_FILE_NAME);
    let manifestContent;
    try {
        manifestContent = await fsReadFileAsync(manifestPath);
    }
    catch (err) {
        throw plugin_discovery_error_1.PluginDiscoveryError.missingManifest(manifestPath, err);
    }
    let manifest;
    try {
        manifest = JSON.parse(manifestContent.toString());
    }
    catch (err) {
        throw plugin_discovery_error_1.PluginDiscoveryError.invalidManifest(manifestPath, err);
    }
    if (!manifest || typeof manifest !== 'object') {
        throw plugin_discovery_error_1.PluginDiscoveryError.invalidManifest(manifestPath, new Error('Plugin manifest must contain a JSON encoded object.'));
    }
    if (!manifest.id || typeof manifest.id !== 'string') {
        throw plugin_discovery_error_1.PluginDiscoveryError.invalidManifest(manifestPath, new Error('Plugin manifest must contain an "id" property.'));
    }
    if (!manifest.version || typeof manifest.version !== 'string') {
        throw plugin_discovery_error_1.PluginDiscoveryError.invalidManifest(manifestPath, new Error(`Plugin manifest for "${manifest.id}" must contain a "version" property.`));
    }
    const expectedKibanaVersion = typeof manifest.kibanaVersion === 'string' && manifest.kibanaVersion
        ? manifest.kibanaVersion
        : manifest.version;
    if (!isVersionCompatible(expectedKibanaVersion, packageInfo.version)) {
        throw plugin_discovery_error_1.PluginDiscoveryError.incompatibleVersion(manifestPath, new Error(`Plugin "${manifest.id}" is only compatible with Kibana version "${expectedKibanaVersion}", but used Kibana version is "${packageInfo.version}".`));
    }
    return {
        id: manifest.id,
        version: manifest.version,
        kibanaVersion: expectedKibanaVersion,
        requiredPlugins: Array.isArray(manifest.requiredPlugins) ? manifest.requiredPlugins : [],
        optionalPlugins: Array.isArray(manifest.optionalPlugins) ? manifest.optionalPlugins : [],
        ui: typeof manifest.ui === 'boolean' ? manifest.ui : false,
    };
}
exports.parseManifest = parseManifest;
/**
 * Checks whether plugin expected Kibana version is compatible with the used Kibana version.
 * @param expectedKibanaVersion Kibana version expected by the plugin.
 * @param actualKibanaVersion Used Kibana version.
 */
function isVersionCompatible(expectedKibanaVersion, actualKibanaVersion) {
    if (expectedKibanaVersion === ALWAYS_COMPATIBLE_VERSION) {
        return true;
    }
    return extractSemVer(actualKibanaVersion) === extractSemVer(expectedKibanaVersion);
}
/**
 * Tries to extract semantic version part from the full version string.
 * @param version
 */
function extractSemVer(version) {
    const semVerMatch = version.match(SEM_VER_REGEX);
    return semVerMatch === null ? version : semVerMatch[0];
}
