"use strict";
/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const plugin_discovery_error_1 = require("./plugin_discovery_error");
const plugin_manifest_parser_1 = require("./plugin_manifest_parser");
const fsReadDir$ = rxjs_1.bindNodeCallback(fs_1.readdir);
const fsStat$ = rxjs_1.bindNodeCallback(fs_1.stat);
/**
 * Tries to discover all possible plugins based on the provided plugin config.
 * Discovery result consists of two separate streams, the one (`plugin$`) is
 * for the successfully discovered plugins and the other one (`error$`) is for
 * all the errors that occurred during discovery process.
 *
 * @param config Plugin config instance.
 * @param packageInfo Kibana package info.
 * @param log Plugin discovery logger instance.
 */
function discover(config, packageInfo, log) {
    log.debug('Discovering plugins...');
    const discoveryResults$ = rxjs_1.merge(processScanDirs$(config.scanDirs, log), processPaths$(config.paths, log)).pipe(operators_1.mergeMap(pluginPathOrError => {
        return typeof pluginPathOrError === 'string'
            ? createPlugin$(pluginPathOrError, packageInfo, log)
            : [pluginPathOrError];
    }), operators_1.shareReplay());
    return {
        plugin$: discoveryResults$.pipe(operators_1.mergeMap(entry => (entry.plugin !== undefined ? [entry.plugin] : []))),
        error$: discoveryResults$.pipe(operators_1.mergeMap(entry => (entry.error !== undefined ? [entry.error] : []))),
    };
}
exports.discover = discover;
/**
 * Iterates over every entry in `scanDirs` and returns a merged stream of all
 * sub-directories. If directory cannot be read or it's impossible to get stat
 * for any of the nested entries then error is added into the stream instead.
 * @param scanDirs List of the top-level directories to process.
 * @param log Plugin discovery logger instance.
 */
function processScanDirs$(scanDirs, log) {
    return rxjs_1.from(scanDirs).pipe(operators_1.mergeMap(dir => {
        log.debug(`Scanning "${dir}" for plugin sub-directories...`);
        return fsReadDir$(dir).pipe(operators_1.mergeMap(subDirs => subDirs.map(subDir => path_1.resolve(dir, subDir))), operators_1.mergeMap(path => fsStat$(path).pipe(
        // Filter out non-directory entries from target directories, it's expected that
        // these directories may contain files (e.g. `README.md` or `package.json`).
        // We shouldn't silently ignore the entries we couldn't get stat for though.
        operators_1.mergeMap(pathStat => (pathStat.isDirectory() ? [path] : [])), operators_1.catchError(err => [wrapError(plugin_discovery_error_1.PluginDiscoveryError.invalidPluginDirectory(path, err))]))), operators_1.catchError(err => [wrapError(plugin_discovery_error_1.PluginDiscoveryError.invalidScanDirectory(dir, err))]));
    }));
}
/**
 * Iterates over every entry in `paths` and returns a stream of all paths that
 * are directories. If path is not a directory or it's impossible to get stat
 * for this path then error is added into the stream instead.
 * @param paths List of paths to process.
 * @param log Plugin discovery logger instance.
 */
function processPaths$(paths, log) {
    return rxjs_1.from(paths).pipe(operators_1.mergeMap(path => {
        log.debug(`Including "${path}" into the plugin path list.`);
        return fsStat$(path).pipe(
        // Since every path is specifically provided we should treat non-directory
        // entries as mistakes we should report of.
        operators_1.mergeMap(pathStat => {
            return pathStat.isDirectory()
                ? [path]
                : rxjs_1.throwError(new Error(`${path} is not a directory.`));
        }), operators_1.catchError(err => [wrapError(plugin_discovery_error_1.PluginDiscoveryError.invalidPluginDirectory(path, err))]));
    }));
}
/**
 * Tries to load and parse the plugin manifest file located at the provided plugin
 * directory path and produces an error result if it fails to do so or plugin manifest
 * isn't valid.
 * @param path Path to the plugin directory where manifest should be loaded from.
 * @param packageInfo Kibana package info.
 * @param log Plugin discovery logger instance.
 */
function createPlugin$(path, packageInfo, log) {
    return rxjs_1.from(plugin_manifest_parser_1.parseManifest(path, packageInfo)).pipe(operators_1.map(manifest => {
        log.debug(`Successfully discovered plugin "${manifest.id}" at "${path}"`);
        return { plugin: { path, manifest } };
    }), operators_1.catchError(err => [wrapError(err)]));
}
/**
 * Wraps `PluginDiscoveryError` into `DiscoveryResult` entry.
 * @param error Instance of the `PluginDiscoveryError` error.
 */
function wrapError(error) {
    return { error };
}
